# -*- coding: utf-8 -*-
#
# Author:        alisue
# Date:            2010/11/07
#
from django.http import HttpResponseForbidden
from django.db.models import Model
from django.contrib.auth.views import redirect_to_login

from utils import generic_permission_check

def permission_required(perm, queryset=None):
    u"""
    Permission check decorator for generic view
    
    Arguments:
        perm     - permission code same format as Django's permission code
                   when you need ObjectPermission feature only, you can cut
                   off `app_label` and `model` See Usage below.
        queryset - QuerySet or Model object for filtering.
        
    Usage:
        @required_permission('app_label.view_model', Model)
        def object_detail(request, *args, **kwargs):
            ...
        
        If you need `ObjectPermission` only, you can cut of app_label and model like below
        
        @required_permission('view', Model)
        def object_detial(request, *args, **kwargs):
            ...
        
    """
    def wrapper(fn):
        def inner(request, *args, **kwargs):
            if queryset and issubclass(queryset, Model):
                qs = queryset.objects.all()
            else:
                qs = queryset
            if not generic_permission_check(qs, perm, request, *args, **kwargs):
                if request.user.is_authenticated():
                    return HttpResponseForbidden()
                else:
                    return redirect_to_login(request.path)
            return fn(request, *args, **kwargs)
        return inner
    return wrapper

# Deprecation Function
import warnings
def permission_check(perm, queryset=None):
    warnings.warn("`permission_check` function is deprecationed. Use `permission_required` insted", DeprecationWarning)
    return permission_required(perm, queryset)



# -*- coding: utf-8 -*-
#
# Created:        2010/11/07
# Author:        alisue
#
from django.conf import settings
from django.db.models import Q, Model
from django.contrib.contenttypes.models import ContentType
from django.contrib.auth.models import Permission

from models import GroupObjectPermission, AnonymousObjectPermission, UserObjectPermission

import warnings

class ObjectPermBackend(object):
    supports_object_permissions = True
    supports_anonymous_user     = True

    def authenticate(self, username, password):
        return None

    def has_perm(self, user_obj, perm, obj=None):
        def exists(qs, perm):
            return qs.filter(permissions__codename=perm).distinct().exists()
        
        if obj is None or not isinstance(obj, Model):
            return False
        
        if not hasattr(obj, settings.OBJECT_PERMISSION_MODIFY_FUNCTION):
            warnings.warn(u"%r doesn't have `%s(self, mediator, created)` function." % (obj.__class__, settings.OBJECT_PERMISSION_MODIFY_FUNCTION))
            
        ct = ContentType.objects.get_for_model(obj)
        
        try:
            perm = perm.split('.')[-1]
        except IndexError:
            return False
        
        # exists check for `perm`
        if not Permission.objects.filter(content_type=ct, codename=perm).exists():
            # Django default permissions have model name at the end.
            _perm = "%s_%s" % (perm, ct.model)
            if not Permission.objects.filter(content_type=ct, codename=_perm).exists():
                permissions = [p.codename for p in Permission.objects.filter(content_type=ct)]
                raise AttributeError("Permission `%s` of `%s` doesn't exists."
                                     " Chose permission from %s or check"
                                     " `http://docs.djangoproject.com/en/dev/topics/auth/#custom-permissions`"
                                     " to create custom permission." % (perm, ct.model, permissions))
            perm = _perm
            
        if user_obj.is_authenticated():
            # try to find from UserObjectPermission first
            qs = UserObjectPermission.objects.filter(content_type=ct, object_id=obj.pk, user=user_obj)
            if not qs.exists():
                # permission for particular user should have priority over permission for all authenticated users
                qs = UserObjectPermission.objects.filter(content_type=ct, object_id=obj.pk, user=None)
            if exists(qs, perm): return True
            qs = GroupObjectPermission.objects.filter(content_type=ct, object_id=obj.pk, group__in= user_obj.groups.all())
            return exists(qs, perm)
        else:
            # try to find from AnonymousObjectPermission
            qs = AnonymousObjectPermission.objects.filter(content_type=ct, object_id=obj.pk)
            return exists(qs, perm)



# -*- coding: utf-8 -*-
#
# Author:        alisue
# Date:            2010/11/29
#
from django.contrib.contenttypes.models import ContentType
from django.contrib.auth.models import User, Group, AnonymousUser, Permission

from models import UserObjectPermission, GroupObjectPermission, AnonymousObjectPermission

class ObjectPermissionMediator(object):
    @classmethod
    def get_permission(cls, obj, to=None):
        u"""
        Get permission for `obj` to `to` with parameter
        
        Attribute:
            obj        - tareget object for permission
            to         - User, Group, AnonymousUser to cotribute permission.
                        `None` mean contribute permission to all authenticated user
        """
        
        ctype = ContentType.objects.get_for_model(obj)
        if isinstance(to, basestring) and to == 'anonymous':
            instance = AnonymousObjectPermission.objects.get_or_create(
                content_type=ctype,
                object_id=obj.pk)[0]
        elif isinstance(to, AnonymousUser):
            instance = AnonymousObjectPermission.objects.get_or_create(
                content_type=ctype,
                object_id=obj.pk)[0]
        elif isinstance(to, Group):
            instance = GroupObjectPermission.objects.get_or_create(
                group=to,
                content_type=ctype,
                object_id=obj.pk)[0]
        else:
            if isinstance(to, basestring):
                to = User.objects.get(username=to)
            instance = UserObjectPermission.objects.get_or_create(
                user=to,
                content_type=ctype,
                object_id=obj.pk)[0]
        return instance
    @classmethod
    def contribute(cls, obj, to=None, permissions=[], clear=False):
        u"""
        Contribute permission for `obj` to `to` with parameter
        
        Attribute:
            obj         - tareget object for permission
            to          - User, Group, AnonymousUser to cotribute permission.
                         `None` mean contribute permission to all authenticated user
            permissions - codename list of permission
            append      - `True` to append `permissions`, default is `False`
        
        Notice:
            this method is super method. Use `manager`, `editor`, `viewer` and `reject`
            for noraml usage.
        """
        ctype = ContentType.objects.get_for_model(obj)
        instance = cls.get_permission(obj, to)
        if clear:
            instance.permissions.clear()
        for perm in permissions:
            try:
                perm = perm.split('.')[-1]
            except IndexError:
                return False
            try:
                permission = Permission.objects.get(
                    content_type=ctype,
                    codename=perm,
                )
            except Permission.DoesNotExist:
                # Django default permissions have model name at the end
                perm = "%s_%s" % (perm, ctype.model)
                permission = Permission.objects.get(
                    content_type=ctype,
                    codename=perm,
                )
            instance.permissions.add(permission)
        instance.save()
        return instance
    @classmethod
    def discontribute(cls, obj, to=None, permissions=[]):
        u"""
        Discontribute permission for `obj` to `to` with parameter
        
        Attribute:
            obj         - tareget object for permission
            to          - User, Group, AnonymousUser to cotribute permission.
                         `None` mean contribute permission to all authenticated user
            permissions - codename list of permission
            append      - `True` to append `permissions`, default is `False`
        
        Notice:
            this method is super method. Use `manager`, `editor`, `viewer` and `reject`
            for noraml usage.
        """
        ctype = ContentType.objects.get_for_model(obj)
        instance = cls.get_permission(obj, to)
        for perm in permissions:
            try:
                perm = perm.split('.')[-1]
            except IndexError:
                return False
            try:
                permission = Permission.objects.get(
                    content_type=ctype,
                    codename=perm,
                )
            except Permission.DoesNotExist:
                # Django default permissions have model name at the end
                perm = "%s_%s" % (perm, ctype.model)
                permission = Permission.objects.get(
                    content_type=ctype,
                    codename=perm,
                )
            instance.permissions.remove(permission)
        instance.save()
        return instance
    @classmethod
    def manager(cls, obj, to, extra_permissions=[]):
        u"""
        Contribute manager permission (can view, change, delete) for `obj` to `to`
        
        Attribute:
            obj        - tareget object for permission
            to         - User, Group, AnonymousUser to cotribute permission.
                        `None` can_view   - `view` permissionOld permission
            can_change - `change` permission
            can_delete - `delete` permissionmean contribute permission to all authenticated user
        """
        return cls.contribute(obj, to, ['view', 'change', 'delete']+list(extra_permissions), clear=True)
    
    @classmethod
    def editor(cls, obj, to, extra_permissions=[]):
        u"""
        Contribute editor permission (can view, change. cannot delete) for `obj` to `to`
        
        Attribute:
            obj        - tareget object for permission
            to         - User, Group, AnonymousUser to cotribute permission.
                        `None` mean contribute permission to all authenticated user
        """
        return cls.contribute(obj, to, ['view', 'change']+list(extra_permissions), clear=True)
    
    @classmethod
    def viewer(cls, obj, to, extra_permissions=[]):
        u"""
        Contribute viewer permission (can view. cannot change, delete) for `obj` to `to`
        
        Attribute:
            obj        - tareget object for permission
            to         - User, Group, AnonymousUser to cotribute permission.
                        `None` mean contribute permission to all authenticated user
        """
        return cls.contribute(obj, to, ['view']+list(extra_permissions), clear=True)
    
    @classmethod
    def reject(cls, obj, to, extra_permissions=[]):
        u"""
        Contribute reject permission (cannot view change, delete) for `obj` to `to`
        
        Attribute:
            obj        - tareget object for permission
            to         - User, Group, AnonymousUser to cotribute permission.
                        `None` mean contribute permission to all authenticated user
        """
        return cls.contribute(obj, to, []+list(extra_permissions), clear=True)


"""
Django 1.2 template tag that supports {% elif %} branches and
'of' operator for checking object permission.

WARNING:
    This template tag assumed that context has 'request' so make sure your TEMPLATE_CONTEXTS is like below
    
        TEMPLATE_CONTEXT_PROCESSORS = (
            "django.core.context_processors.auth",           # This one is required
            "django.core.context_processors.debug",
            "django.core.context_processors.i18n",
            "django.core.context_processors.media",
            "django.core.context_processors.request",        # This one is required
        )
        
Usage:

    {% if 'blogs.add_entry' of None or user.is_staff %}
        You can add post
    {% elif 'blogs.change_entry' of object or 'blogs.delete_entry' of object %}
        You can update/delete this entry
    {% endif %}
    
"""
from django import template
from django.template import Node, VariableDoesNotExist
from django.template.smartif import infix
from django.template.smartif import IfParser
from django.template.smartif import OPERATORS as _OPERATORS
from django.template.defaulttags import TemplateLiteral

register = template.Library()

OPERATORS = dict(_OPERATORS,
    of=infix(10, lambda context, x, y: template.resolve_variable('request', context).user.has_perm(x.eval(context), y.eval(context)))
)

class ObjectPermissionIfParser(IfParser):
    def translate_token(self, token):
        try:
            op = OPERATORS[token]
        except (KeyError, TypeError):
            return self.create_var(token)
        else:
            return op()
class TemplateObjectPermissionIfParser(ObjectPermissionIfParser):
    error_class = template.TemplateSyntaxError

    def __init__(self, parser, *args, **kwargs):
        self.template_parser = parser
        return super(TemplateObjectPermissionIfParser, self).__init__(*args, **kwargs)

    def create_var(self, value):
        return TemplateLiteral(self.template_parser.compile_filter(value), value)
    
class IfBranch(object):
    def __init__(self, var, node_list):
        self.var = var
        self.node_list = node_list

class IfNode(Node):
    def __init__(self, branches):
        self.branches = branches

    def __repr__(self):
        return "<If node>"

    def __iter__(self):
        for n in self.branches:
            for node in n:
                yield node

    def render(self, context):
        for n in self.branches:
            var = n.var
            if var != True:
                try:
                    var = var.eval(context)
                except VariableDoesNotExist:
                    var = None
            if var:
                return n.node_list.render(context)
                break
        return ""

@register.tag('if')
def do_if(parser, token):
    class Enders(list):
        def __init__(self, endtag):
            self.endtag = endtag
        def __contains__(self, val):
            return val.startswith('elif') or val in ('else', self.endtag)
    
    name = contents = token.split_contents()[0]
    endtag = "end%s" % name
    enders = Enders(endtag)
    branches = []
    
    while True:
        contents = token.split_contents()
        bits = contents[1:]
        if contents[0] in (name, "elif"):
            var = TemplateObjectPermissionIfParser(parser, bits).parse()
            nodelist = parser.parse(enders)
            next_token = parser.next_token()
            branches.append(IfBranch(var, nodelist))
            token = next_token
        elif token.contents == 'else':
            nodelist = parser.parse((endtag,))
            parser.delete_first_token()
            branches.append(IfBranch(True, nodelist))
            break
        elif token.contents == endtag:
            break

    return IfNode(branches)
register.tag("ifhsp", do_if)


# -*- coding: utf-8 -*-
#
# Author:        alisue
# Date:            2010/11/07
#
from django.conf import settings

settings.OBJECT_PERMISSION_MODIFY_FUNCTION = getattr(settings, 'OBJECT_PERMISSION_MODIFY_FUNCTION', 'modify_object_permission')
settings.OBJECT_PERMISSION_MODIFY_M2M_FUNCTION = getattr(settings, 'OBJECT_PERMISSION_MODIFY_M2M_FUNCTION', 'modify_object_permission_m2m')
from mediators import ObjectPermissionMediator

# Regist templatetags for ObjectPermission
#from django.template import add_to_builtins
#add_to_builtins('object_permission.templatetags.object_permission_tags')

# Automatically call `modify_permission` of all model
from django.db.models.signals import post_save, m2m_changed
def _post_save_callback(sender, instance, created, **kwargs):
    if hasattr(instance, settings.OBJECT_PERMISSION_MODIFY_FUNCTION):
        fn = getattr(instance, settings.OBJECT_PERMISSION_MODIFY_FUNCTION)
        fn(mediator=ObjectPermissionMediator, created=created)
def _m2m_changed_callback(sender, instance, action, reverse, model, pk_set, **kwargs):
    if action in ('post_add', 'post_remove') and hasattr(instance, settings.OBJECT_PERMISSION_MODIFY_M2M_FUNCTION):
        fn = getattr(instance, settings.OBJECT_PERMISSION_MODIFY_M2M_FUNCTION)
        fn(mediator=ObjectPermissionMediator, sender=sender, model=model, pk_set=pk_set, removed=action == 'post_remove')
post_save.connect(_post_save_callback)
m2m_changed.connect(_m2m_changed_callback)


# -*- coding: utf-8 -*-
#
# Author:        alisue
# Date:            2010/11/30
#
from django.conf import settings
from django.db.models.loading import get_models
from django.contrib.contenttypes.models import ContentType
from django.core.management.base import BaseCommand, CommandError
from django.core.exceptions import ImproperlyConfigured

from ...mediators import ObjectPermissionMediator

class Command(BaseCommand):
    help = u"Remodify object_permission of each model using model's `modify_object_permission(self, mediator, created)` function."

    def handle(self, *app_labels, **options):
        # Historyを停止
        settings.HISTORY_ENABLE = False
        from django.db import models
        if not app_labels:
            output = [self.handle_noargs(**options)]
        else:
            try:
                app_list = [models.get_app(app_label) for app_label in app_labels]
            except (ImproperlyConfigured, ImportError), e:
                raise CommandError("%s. Are you sure your INSTALLED_APPS setting is correct?" % e)
            output = []
            for app in app_list:
                app_output = self.handle_app(app, **options)
                if app_output:
                    output.append(app_output)
        # Historyを再開
        settings.HISTORY_ENABLE = True
        return '\n'.join(output)
    
    def handle_app(self, app, **options):
        u"Remodify object_permission of each model using model's `modify_object_permission(self, mediator, created)` function."
        output = []
        model_list = get_models(app)
        for model in model_list:
            _output = self._remodify_object_permission(model, **options)
            if _output:
                output.append(_output)
        return "\n".join(output)
    
    def handle_noargs(self, **options):
        output = []
        content_types = ContentType.objects.all()
        for ct in content_types:
            model = ct.model_class()
            _output = self._remodify_object_permission(model, **options)
            if _output:
                output.append(_output)
        return "\n".join(output)
    
    def _remodify_object_permission(self, model, **options):
        NAME = settings.OBJECT_PERMISSION_MODIFY_FUNCTION
        NAME_M2M = settings.OBJECT_PERMISSION_MODIFY_M2M_FUNCTION
        output = []
        if not hasattr(model, NAME) and not hasattr(model, NAME_M2M):
            if options['verbosity'] == '2':
                output.append("Skipped: %s doesn't have `%s` and `%s` function." % (model, NAME, NAME_M2M))
        else:
            has = hasattr(model, NAME)
            has_m2m = hasattr(model, NAME_M2M)
            count = model.objects.count()
            for obj in model.objects.all():
                if has:
                    fn = getattr(obj, NAME)
                    fn(mediator=ObjectPermissionMediator, created=False)
                if has_m2m:
                    fn_m2m = getattr(obj, NAME_M2M)
                    for field in obj._meta.many_to_many:
                        sender = field.rel.through
                        model = field.rel.to
                        pk_set = [to.pk for to in getattr(obj, field.attname).all()]
                        fn_m2m(mediator=ObjectPermissionMediator, sender=sender, model=model, pk_set=pk_set, removed=False)
            if options['verbosity'] != '0':
                output.append("Modified: %s's object_permission has remodified (%d)" % (model, count))
        return "\n".join(output)


# -*- coding: utf-8 -*-
#
# Author:        alisue
# Date:            2010/11/07
#
from django.http import HttpResponseRedirect
from django.core.urlresolvers import reverse
from django.contrib import admin
from django.contrib.contenttypes.generic import GenericTabularInline
from django.contrib.flatpages.models import FlatPage
from django.contrib.flatpages.admin import FlatPageAdmin as FPAdmin
from django.contrib import messages
from django.core.exceptions import PermissionDenied
from models import UserObjectPermission, GroupObjectPermission, AnonymousObjectPermission

#class ObjectPermissionMixin(object):
#    def has_change_permission(self, request, obj=None):
#        opts = self.opts
#        return request.user.has_perm(opts.app_label + '.' + opts.get_change_permission(), obj)
#    def has_delete_permission(self, request, obj=None):
#        opts = self.opts
#        return request.user.has_perm(opts.app_label + '.' + opts.get_delete_permission(), obj)
#    
#class UserObjectPermissionInline(GenericTabularInline):
#    model = UserObjectPermission
#    extra = 1
#    raw_id_fields = ['user']
#
#class GroupObjectPermissionInline(GenericTabularInline):
#    model = GroupObjectPermission
#    extra = 1
#    raw_id_fields = ['group']
#
#class AnonymousObjectPermissionInline(GenericTabularInline):
#    model = AnonymousObjectPermission
#    extra = 1
#    max_num = 1
#    can_delete = False
#
#class FlatPageAdmin(ObjectPermissionMixin, FPAdmin):
#    inlines = FPAdmin.inlines + [UserObjectPermissionInline, GroupObjectPermissionInline, AnonymousObjectPermissionInline]
#    
#    def change_view(self, request, *args, **kwargs):
#        try:
#            return super(FlatPageAdmin, self).change_view(request, *args, **kwargs)
#        except PermissionDenied, e:
#            messages.add_message(request, messages.ERROR, u"You don't have the necessary permissions!")
#            return HttpResponseRedirect(reverse('admin:flatpages_flatpage_changelist'))
#
#admin.site.unregister(FlatPage)
#admin.site.register(FlatPage, FlatPageAdmin)

class ObjectPermissionAdmin(admin.ModelAdmin):
    def _get_permissions(self, obj):
        permissions = []
        for permission in obj.permissions.all():
            permissions.append(permission.codename)
        if not permissions:
            return "-"
        return "<br />".join(permissions)
    _get_permissions.allow_tags = True
    _get_permissions.short_description = "Permissions"
class UserObjectPermissionAdmin(ObjectPermissionAdmin):
    list_display    = ('content_type', 'content_object', 'user', '_get_permissions')
    list_filter     = ('content_type', 'user', 'permissions')
    search_fields   = ('permissions', 'user',)
class GroupObjectPermissionAdmin(ObjectPermissionAdmin):
    list_display    = ('content_type', 'content_object', 'group', '_get_permissions')
    list_filter     = ('content_type', 'group', 'permissions')
    search_fields   = ('permissions', 'group',)
class AnonymousObjectPermissionAdmin(ObjectPermissionAdmin):
    list_display    = ('content_type', 'content_object', '_get_permissions')
    list_filter     = ('content_type', 'permissions')
    search_fields   = ('permissions',)
admin.site.register(UserObjectPermission, UserObjectPermissionAdmin)
admin.site.register(GroupObjectPermission, GroupObjectPermissionAdmin)
admin.site.register(AnonymousObjectPermission, AnonymousObjectPermissionAdmin)


# -*- coding: utf-8 -*-
#
# Author:        alisue
# Date:            2010/11/07
#
from django.test import TestCase
from django.contrib.flatpages.models import FlatPage
from django.contrib.contenttypes.models import ContentType
from django.contrib.auth.models import User, Group, AnonymousUser

from mediators import ObjectPermissionMediator

class UserObjectPermBackendTest(TestCase):
    def setUp(self):
        self.james = User.objects.create_user('james', 'james@example.com', 'james')
        self.emily = User.objects.create_user('emily', 'emily@example.com', 'emily')
        self.peter = User.objects.create_user('peter', 'emily@example.com', 'peter')
        self.fread = AnonymousUser()
        self.page = FlatPage.objects.create(url='/test/', title='test', content='James can see this content but emily.')
        # for checking permission for all authenticated user
        ObjectPermissionMediator.viewer(self.page, None)
        # for checking permission for particular user
        ObjectPermissionMediator.manager(self.page, self.james)
        ObjectPermissionMediator.editor(self.page, self.emily)
        # for checking overwrite permission for particular user
        ObjectPermissionMediator.reject(self.page, self.peter)
        
    def test_permission(self):
        # James has view, change, delete
        self.assertTrue(self.james.has_perm('flatpages.view_flatpage', self.page))
        self.assertTrue(self.james.has_perm('flatpages.change_flatpage', self.page))
        self.assertTrue(self.james.has_perm('flatpages.delete_flatpage', self.page))
        # Emliy has view, change but delete
        self.assertTrue(self.emily.has_perm('flatpages.view_flatpage', self.page))
        self.assertTrue(self.emily.has_perm('flatpages.change_flatpage', self.page))
        self.assertTrue(not self.emily.has_perm('flatpages.delete_flatpage', self.page))
        # Peter never has any permission
        self.assertTrue(not self.peter.has_perm('flatpages.view_flatpage', self.page))
        self.assertTrue(not self.peter.has_perm('flatpages.change_flatpage', self.page))
        self.assertTrue(not self.peter.has_perm('flatpages.delete_flatpage', self.page))
        # Fread never has any permission because he is not authenticated
        self.assertTrue(not self.fread.has_perm('flatpages.view_flatpage', self.page))
        self.assertTrue(not self.fread.has_perm('flatpages.change_flatpage', self.page))
        self.assertTrue(not self.fread.has_perm('flatpages.delete_flatpage', self.page))

class GroupObjectPermBackendTest(TestCase):
    def setUp(self):
        self.james = User.objects.create_user('james', 'james@example.com', 'james')
        self.emily = User.objects.create_user('emily', 'emily@example.com', 'emily')
        self.peter = User.objects.create_user('peter', 'emily@example.com', 'peter')
        self.fread = AnonymousUser()
        self.manager = Group.objects.create(name='manager')
        self.editor = Group.objects.create(name='editor')
        self.james.groups.add(self.manager)
        self.james.groups.add(self.editor)
        self.emily.groups.add(self.editor)
        self.page = FlatPage.objects.create(url='/test/', title='test', content='James can see this content but emily.')
        # for checking permission for all authenticated user
        ObjectPermissionMediator.viewer(self.page, None)
        # for checking permission for particular group
        ObjectPermissionMediator.manager(self.page, self.manager)
        ObjectPermissionMediator.editor(self.page, self.editor)
        
    def test_permission(self):
        # James has view, change, delete
        self.assertTrue(self.james.has_perm('flatpages.view_flatpage', self.page))
        self.assertTrue(self.james.has_perm('flatpages.change_flatpage', self.page))
        self.assertTrue(self.james.has_perm('flatpages.delete_flatpage', self.page))
        # Emliy has view, change but delete
        self.assertTrue(self.emily.has_perm('flatpages.view_flatpage', self.page))
        self.assertTrue(self.emily.has_perm('flatpages.change_flatpage', self.page))
        self.assertTrue(not self.emily.has_perm('flatpages.delete_flatpage', self.page))
        # Peter has view
        self.assertTrue(self.peter.has_perm('flatpages.view_flatpage', self.page))
        self.assertTrue(not self.peter.has_perm('flatpages.change_flatpage', self.page))
        self.assertTrue(not self.peter.has_perm('flatpages.delete_flatpage', self.page))
        # Fread never has any permission because he is not authenticated
        self.assertTrue(not self.fread.has_perm('flatpages.view_flatpage', self.page))
        self.assertTrue(not self.fread.has_perm('flatpages.change_flatpage', self.page))
        self.assertTrue(not self.fread.has_perm('flatpages.delete_flatpage', self.page))

class AnonymousObjectPermBackendTest(TestCase):
    def setUp(self):
        self.james = User.objects.create_user('james', 'james@example.com', 'james')
        self.fread = AnonymousUser()
        self.page = FlatPage.objects.create(url='/test/', title='test', content='James can see this content but emily.')
        # for checking permission for all authenticated user
        ObjectPermissionMediator.viewer(self.page, None)
        # for checking permission for anonymous user
        ObjectPermissionMediator.manager(self.page, 'anonymous')
        
    def test_permission(self):
        # James has view
        self.assertTrue(self.james.has_perm('flatpages.view_flatpage', self.page))
        self.assertTrue(not self.james.has_perm('flatpages.change_flatpage', self.page))
        self.assertTrue(not self.james.has_perm('flatpages.delete_flatpage', self.page))
        # Fread has view, change, delete because he is anonymous user
        self.assertTrue(self.fread.has_perm('flatpages.view_flatpage', self.page))
        self.assertTrue(self.fread.has_perm('flatpages.change_flatpage', self.page))
        self.assertTrue(self.fread.has_perm('flatpages.delete_flatpage', self.page))


# -*- coding: utf-8 -*-
#
# Created:        2010/11/07
# Author:        alisue
#
# ref: http://djangoadvent.com/1.2/object-permissions/
#
from django.db import models
from django.contrib.auth.models import User, Group, Permission
from django.contrib.contenttypes import generic
from django.contrib.contenttypes.models import ContentType
from django.utils.translation import ugettext_lazy as _
        
class BaseObjectPermission(models.Model):
    """
    Permission model for perticular object
    """
    content_type    = models.ForeignKey(ContentType, verbose_name=_('content type'), null=True)
    object_id       = models.PositiveIntegerField(_('object id'), null=True)
    content_object  = generic.GenericForeignKey()
    
    permissions     = models.ManyToManyField(Permission, verbose_name=_('permissions'), null=True)
    
    class Meta:
        abstract    = True
        
class GroupObjectPermission(BaseObjectPermission):
    """
    Object permission model for group
    """
    group   = models.ForeignKey(Group, verbose_name=_('group'))
    
    class Meta:
        ordering            = ('content_type', 'group')
        unique_together     = ('content_type', 'object_id', 'group')
        verbose_name        = _('group object permission')
        verbose_name_plural = _('group object permissions')
        
class AnonymousObjectPermission(BaseObjectPermission):
    """
    Object permission model for anonymous user
    """
    class Meta:
        ordering            = ('content_type',)
        unique_together     = ('content_type', 'object_id')
        verbose_name        = _('anonymous object permission')
        verbose_name_plural = _('anonymous object permissions')

class UserObjectPermission(BaseObjectPermission):
    """
    Object permission model for user
    """
    user    = models.ForeignKey(User, blank=True, null=True, verbose_name=_('user'), help_text=_('`None` for all authenticated user'))
    
    class Meta:
        ordering            = ('content_type', 'user')
        unique_together     = ('content_type', 'object_id', 'user')
        verbose_name        = _('user object permission')
        verbose_name_plural = _('user object permissions')


# -*- coding: utf-8 -*-
#
# Author:        alisue
# Date:            2010/11/07
#
from django.shortcuts import get_object_or_404

#
# Notice:
#    DO NOT USE `generic_permission_check` for normal generic view.
#    there is `permission_check` decorator on `libwaz.contrib.object_permission.decorators`
#    and using `generic_permission_check` approach make no sence.
#    `generic_permission_check` approach only make sence when you need to filtering
#    queryset because otherwise `get` method return multiple objects.
#
#    Example usage:
#    You have a Blog application and you are using `date_based` filtering with `author` field.
#    Then you need to filtering queryset with `author` before hand queryset to permission check
#    
def generic_permission_check(queryset, perm, request, *args, **kwargs):
    u"""
    Generic permission check
    
    Arguments:
        queryset    - queryset of object
        perm        - permission string
        request     - HttpRequest
        *args       - *args for view function
        **kwargs    - **kwargs for view function
    
    Usage:
        from django.http import HttpResponseForbidden
        from django.views.generic import list_detail
        from object_permission.utils import generic_permission_check
        
        def permission_required(perm, model):
            def wrapper(fn):
                def inner(request, *args, **kwargs):
                    # Filtering queryset with `author`
                    queryset = model.objects.filter(author=kwargs['author'])
                    if not generic_permission_check(queryset, perm, request, *args, **kwargs):
                        return HttpResponseForbidden()
                    return fn(request, *args, **kwargs)
                return inner
            return wrapper
        
        @permission_required('blogs.view_blog', Entry)
        def object_detail(request, *args, **kwargs):
            kwargs['queryset'] = kwargs['queryset'].filter(author__username=kwargs['author'])
            return list_detail.object_list(request, *args, **kwargs)
    
    """
    if queryset is None:
        return request.user.has_perm(perm)
    if 'year' in kwargs and 'month' in kwargs and 'day' in kwargs and 'slug' in kwargs:
        obj = get_object_or_404(queryset,
            publish_at__year=kwargs['year'],
            publish_at__month=kwargs['month'],
            publish_at__day=kwargs['day'],
            title=kwargs['slug'])
    elif 'object_id' in kwargs:
        # For method based generic view
        obj = get_object_or_404(queryset, pk=kwargs['object_id'])
    elif 'pk' in kwargs:
        # For class based generic view
        obj = get_object_or_404(queryset, pk=kwargs['pk'])
    elif 'slug' in kwargs:
        slug_field = kwargs.get('slug_field', 'slug')
        obj = get_object_or_404(queryset, **{slug_field: kwargs['slug']})
    else:
        return request.user.has_perm(perm)
    return request.user.has_perm(perm, obj)